@@ -1,6 +1,6 @@
 /* Functions for handling the sentence structure.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,18 @@
 #include "goal.h"
 #include "vec.h"
 
+//#define LETTERS
+
+#ifdef LETTERS
 static char * sen_values[6] = {" ", "T", "F", "*", "?", "#"};
+#else
+static char * sen_values[6] = {GTK_STOCK_MEDIA_STOP,
+			       GTK_STOCK_APPLY,
+			       GTK_STOCK_CANCEL,
+			       GTK_STOCK_STOP,
+			       GTK_STOCK_SPELL_CHECK,
+			       GTK_STOCK_SELECT_COLOR};
+#endif
 
 // GTextCharPredicate for determining the location of the comment.
 static gboolean
@@ -81,6 +92,7 @@ sentence_init (sen_data * sd, sen_parent * sp, item_t * fcs)
   sen->value_type = VALUE_TYPE_BLANK;
   sen->selected = 0;
   sen->sexpr = NULL;
+  sen->font_resizing = 0;
 
   sen->indices = (int *) calloc (sen->depth + 1, sizeof (int));
   CHECK_ALLOC (sen->indices, NULL);
@@ -95,7 +107,8 @@ sentence_init (sen_data * sd, sen_parent * sp, item_t * fcs)
 
       fcs_sen = fcs->value;
 
-      index_copy_end = (fcs_sen->depth < sen->depth) ? fcs_sen->depth : sen->depth;
+      index_copy_end = (fcs_sen->depth < sen->depth)
+	? fcs_sen->depth : sen->depth;
 
       for (i = 0; i < index_copy_end; i++)
 	sen->indices[i] = fcs_sen->indices[i];
@@ -113,13 +126,24 @@ sentence_init (sen_data * sd, sen_parent * sp, item_t * fcs)
 
   sen->rule = sd->rule;
 
+  if (sen->rule != -1)
+    {
+      gtk_label_set_text (GTK_LABEL (sen->rule_box),
+			  rules_list[sen->rule]);
+    }
+
   if (sd->text)
     {
-      sen->text = (unsigned char *) calloc (strlen (sd->text) + 1, sizeof (char));
+      sen->text = (unsigned char *) calloc (strlen (sd->text) + 1,
+					    sizeof (char));
       CHECK_ALLOC (sen->text, NULL);
       strcpy (sen->text, sd->text);
-      gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (sen->entry)),
-				(const char *) sd->text, -1);
+      ret = sentence_paste_text (sen);
+      if (ret == -1)
+	return NULL;
+
+      if (ret == -2)
+	return NULL;
     }
   else
     {
@@ -166,7 +190,8 @@ sentence_init (sen_data * sd, sen_parent * sp, item_t * fcs)
   if (sd->file)
     {
       unsigned int file_len = strlen ((const char *) sd->file);
-      sen->file = (unsigned char *) calloc (file_len + 1, sizeof (char));
+      sen->file = (unsigned char *) calloc (file_len + 1,
+					    sizeof (char));
       CHECK_ALLOC (sen->file, NULL);
       strcpy (sen->file, sd->file);
     }
@@ -182,9 +207,9 @@ sentence_init (sen_data * sd, sen_parent * sp, item_t * fcs)
   sentence_set_font (sen, sen->parent->font);
 
   if (!sd->premise)
-    sentence_set_bg (sen, BG_COLOR_CYAN);
+    sentence_set_bg (sen, BG_COLOR_CONC);
   else
-    sentence_set_bg (sen, BG_COLOR_WHITE);
+    sentence_set_bg (sen, BG_COLOR_DEFAULT);
 
   sentence_connect_signals (sen);
 
@@ -215,43 +240,63 @@ sentence_gui_init (sentence * sen)
   if (sen->depth > 0)
     {
       widget = gtk_label_new (NULL);
-      g_object_set (G_OBJECT (widget), "width-chars", 4 * sen->depth, NULL);
+      g_object_set (G_OBJECT (widget), "width-chars", 4 * sen->depth,
+		    NULL);
     }
 
   sen->entry = gtk_text_view_new ();
 
+
+#ifdef LETTERS
   sen->value = gtk_label_new (NULL);
   gtk_label_set_justify (GTK_LABEL (sen->value), GTK_JUSTIFY_FILL);
   gtk_label_set_width_chars (GTK_LABEL (sen->value), 2);
+#else
+  sen->value = gtk_image_new_from_stock (sen_values[0], GTK_ICON_SIZE_MENU);
+  //sen->value = gtk_image_new ();
+#endif
 
-  gtk_box_pack_start (GTK_BOX (sen->panel), sen->eventbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (sen->panel), sen->eventbox, FALSE,
+		      FALSE, 0);
   
   if (sen->depth > 0)
     {
-      gtk_box_pack_start (GTK_BOX (sen->panel), widget, FALSE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (sen->panel), widget, FALSE, FALSE,
+			  0);
     }
 
-  gtk_box_pack_start (GTK_BOX (sen->panel), sen->entry, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (sen->panel), sen->value, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (sen->panel), sen->entry, TRUE, TRUE,
+		      0);
+
+  gtk_box_pack_start (GTK_BOX (sen->panel), sen->value, FALSE, FALSE,
+		      0);
+
+  sen->rule_box = gtk_label_new (NULL);
+  gtk_label_set_justify (GTK_LABEL (sen->rule_box), GTK_JUSTIFY_FILL);
+  gtk_label_set_width_chars (GTK_LABEL (sen->rule_box), 2);
+
+  gtk_box_pack_start (GTK_BOX (sen->panel), sen->rule_box, FALSE,
+		      FALSE, 0);
 
   sen->mark = NULL;
 
   GtkTextTag * tag;
   GtkTextTagTable * table;
 
-  table = gtk_text_buffer_get_tag_table (gtk_text_view_get_buffer
-					 (GTK_TEXT_VIEW (sen->entry)));
+  table =
+    gtk_text_buffer_get_tag_table (gtk_text_view_get_buffer
+				   (GTK_TEXT_VIEW (sen->entry)));
 
   tag = gtk_text_tag_new ("hilight");
   g_object_set (G_OBJECT (tag),
-		"background-gdk", the_app->bg_colors[BG_COLOR_EMERALD],
+		"background-gdk", the_app->bg_colors[BG_COLOR_GOOD],
 		NULL);
 
   gtk_text_tag_table_add (table, tag);
 
   tag = gtk_text_tag_new ("negative");
   g_object_set (G_OBJECT (tag),
-		"background-gdk", the_app->bg_colors[BG_COLOR_CRIMSON],
+		"background-gdk", the_app->bg_colors[BG_COLOR_BAD],
 		NULL);
 
   gtk_text_tag_table_add (table, tag);
@@ -263,7 +308,6 @@ sentence_gui_init (sentence * sen)
  *  output:
  *    none.
  */
-
 void
 sentence_destroy (sentence * sen)
 {
@@ -336,6 +380,13 @@ sentence_copy_to_data (sentence * sen)
       CHECK_ALLOC (refs, NULL);
     }
 
+  if (sen->text)
+    free (sen->text);
+
+  sen->text = sentence_copy_text (sen);
+  if (!sen->text)
+    return NULL;
+
   refs[i] = -1;
 
   sd = sen_data_init (sen->line_num, sen->rule,
@@ -388,7 +439,8 @@ sentence_set_line_no (sentence * sen, int new_line_no)
   if (sp_chk != (int) label_len + 1)
     {
       //There is an error, so exit the program.
-      fprintf (stderr, "Print Error - Unable to print the correct characters to a string.\n");
+      fprintf (stderr, "Print Error - \
+Unable to print the correct characters to a string.\n");
       return -1;
     }
 
@@ -409,9 +461,68 @@ sentence_set_line_no (sentence * sen, int new_line_no)
 void
 sentence_set_font (sentence * sen, int font)
 {
+  int font_size;
+
+  sen->font_resizing = 1;
+  font_size = pango_font_description_get_size (the_app->fonts[font]);
+  font_size /= PANGO_SCALE;
+
+  sentence_resize_text (sen, font_size);
   LABEL_SET_FONT (sen->line_no, the_app->fonts[font]);
   ENTRY_SET_FONT (sen->entry, the_app->fonts[font]);
+#ifdef LETTERS
   LABEL_SET_FONT (sen->value, the_app->fonts[font]);
+#else
+  //gtk_image_set_from_stock (GTK_WIDGET (sen->value), sen_values[sen->value_type], sen_value_sizes[font]);
+#endif
+  LABEL_SET_FONT (sen->rule_box, the_app->fonts[font]);
+
+  sen->font_resizing = 0;
+}
+
+/* Resizes the pixbufs of a sentence.
+ *  input:
+ *    sen - the sentence to be resized.
+ *    new_size - the font size to which to set the sentence.
+ *  output:
+ *    0 on success.
+ */
+int
+sentence_resize_text (sentence * sen, int new_size)
+{
+  GtkTextIter iter;
+  GtkTextBuffer * buffer;
+
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (sen->entry));
+  gtk_text_buffer_get_start_iter (buffer, &iter);
+
+  while (!gtk_text_iter_is_end (&iter))
+    {
+      GdkPixbuf * pix;
+      pix = gtk_text_iter_get_pixbuf (&iter);
+      if (pix)
+	{
+	  GdkPixbuf * new_pix, * old_pix;
+	  char * val;
+	  val = (char *) g_object_get_data (G_OBJECT (pix), _("conn"));
+
+	  new_pix = sen_parent_get_conn_by_type (sen->parent,
+						 val);
+
+	  GtkTextIter next;
+	  next = iter;
+	  gtk_text_iter_forward_char (&next);
+	  gtk_text_buffer_delete (buffer, &iter, &next);
+	  gtk_text_buffer_insert_pixbuf (buffer, &iter,
+					 new_pix);
+	}
+      else
+	{
+	  gtk_text_iter_forward_char (&iter);
+	}
+    }
+
+  return 0;
 }
 
 /* Sets the background color of a sentence.
@@ -426,10 +537,13 @@ sentence_set_bg (sentence * sen, int bg_color)
 {
   COLOR_TYPE inv;
   INVERT (the_app->bg_colors[bg_color], inv);
-  gtk_widget_modify_bg (sen->entry, GTK_STATE_NORMAL, the_app->bg_colors[bg_color]);
-  gtk_widget_modify_base (sen->entry, GTK_STATE_NORMAL, the_app->bg_colors[bg_color]);
+  gtk_widget_modify_bg (sen->entry, GTK_STATE_NORMAL,
+			the_app->bg_colors[bg_color]);
+  gtk_widget_modify_base (sen->entry, GTK_STATE_NORMAL,
+			  the_app->bg_colors[bg_color]);
   gtk_widget_modify_bg (sen->entry, GTK_STATE_SELECTED, inv);
   sen->bg_color = bg_color;
+  free (inv);
 }
 
 /* Sets the evaluation value of a sentence.
@@ -443,7 +557,11 @@ void
 sentence_set_value (sentence * sen, int value_type)
 {
   sen->value_type = value_type;
+#ifdef LETTERS
   gtk_label_set_text (GTK_LABEL (sen->value), sen_values[value_type]);
+#else
+  gtk_image_set_from_stock (GTK_IMAGE (sen->value), sen_values [value_type], GTK_ICON_SIZE_MENU);
+#endif
 }
 
 /* Connects the callback signals to a sentence.
@@ -490,7 +608,7 @@ sentence_in (sentence * sen)
   if (!sen->premise)
     {
       // Set the background color to cyan.
-      sentence_set_bg (sen, BG_COLOR_CYAN);
+      sentence_set_bg (sen, BG_COLOR_CONC);
 
       // Toggle the rule, if one exists.
       if (sen->rule != -1)
@@ -520,7 +638,8 @@ sentence_in (sentence * sen)
 	  sentence_set_reference (SENTENCE (r_itr->value), 1, entire);
 	}
     }
-  else if (sp->type == SEN_PARENT_TYPE_PROOF && the_app->rt->toggled != -1)
+  else if (sp->type == SEN_PARENT_TYPE_PROOF
+	   && the_app->rt->toggled != -1)
     {
       the_app->rt->user = 0;
       TOGGLE_BUTTON (the_app->rt->rules[the_app->rt->toggled]);
@@ -557,7 +676,7 @@ sentence_out (sentence * sen)
     {
       // Reset the background color.
       // Reset the background color of the references.
-      sentence_set_bg (sen, BG_COLOR_WHITE);
+      sentence_set_bg (sen, BG_COLOR_DEFAULT);
 
       item_t * ref_itr = sen->refs->head;
       for (; ref_itr; ref_itr = ref_itr->next)
@@ -739,6 +858,12 @@ select_sentence (sentence * sen)
   return 0;
 }
 
+/* Gets the index of a text iter overall.
+ *  input:
+ *    iter - the text iter of which to get the index.
+ *  output:
+ *    The index in the buffer of the text iter.
+ */
 int
 get_index (GtkTextIter * iter)
 {
@@ -753,13 +878,21 @@ get_index (GtkTextIter * iter)
     {
       GtkTextIter tmp_itr;
       gtk_text_buffer_get_iter_at_line (buffer, &tmp_itr, i);
-      ret += gtk_text_iter_get_bytes_in_line (&tmp_itr);
+      ret += gtk_text_iter_get_chars_in_line (&tmp_itr);
     }
 
-  ret += gtk_text_iter_get_line_index (iter);
+  ret += gtk_text_iter_get_line_offset (iter);
   return ret;
 }
 
+/* Gets the iterator of the text buffer at the given index.
+ *  input:
+ *    buffer - the text buffer.
+ *    iter - a pointer to the text iterator to get the iterator.
+ *    index - the index of which to get the iterator.
+ *  output:
+ *    none.
+ */
 void
 get_iter_at_index (GtkTextBuffer * buffer, GtkTextIter * iter, int index)
 {
@@ -772,16 +905,18 @@ get_iter_at_index (GtkTextBuffer * buffer, GtkTextIter * iter, int index)
       int tmp;
 
       gtk_text_buffer_get_iter_at_line (buffer, &tmp_itr, i);
-      tmp = gtk_text_iter_get_bytes_in_line (&tmp_itr);
+      tmp = gtk_text_iter_get_chars_in_line (&tmp_itr);
       if (tmp + offset >= index)
 	break;
       offset += tmp;
       i++;
     }
 
-  gtk_text_buffer_get_iter_at_line_index (buffer, iter, i, index - offset);
+  gtk_text_buffer_get_iter_at_line_offset (buffer, iter, i,
+					   index - offset);
 }
 
+
 /* Processes a key press for a sentence.
  *  input:
  *    sen - the sentence being processed.
@@ -801,49 +936,57 @@ sentence_key (sentence * sen, int key, int ctrl)
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (sen->entry));
 
   gtk_text_buffer_get_bounds (buffer, &start, &end);
-  gtk_text_buffer_remove_tag_by_name (buffer, "hilight", &start, &end);
-  gtk_text_buffer_remove_tag_by_name (buffer, "negative", &start, &end);
+  gtk_text_buffer_remove_tag_by_name (buffer, "hilight", &start,
+				      &end);
+  gtk_text_buffer_remove_tag_by_name (buffer, "negative", &start,
+				      &end);
 
   int tmp_pos, offset;
 
   if (ctrl)
     {
       char * insert_char = NULL;
+      GdkPixbuf * pixbuf = NULL;
+      int font_size;
+
+      font_size =
+	pango_font_description_get_size (the_app->fonts[sen->parent->font]);
+      font_size /= PANGO_SCALE;
 
       switch (key)
 	{
 	case GDK_KEY_7:
-	  insert_char = AND;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, AND);
 	  break;
 	case GDK_KEY_backslash:
-	  insert_char = OR;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, OR);
 	  break;
 	case GDK_KEY_grave:
-	  insert_char = NOT;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, NOT);
 	  break;
 	case GDK_KEY_4:
-	  insert_char = CON;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, CON);
 	  break;
 	case GDK_KEY_5:
-	  insert_char = BIC;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, BIC);
 	  break;
 	case GDK_KEY_2:
-	  insert_char = UNV;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, UNV);
 	  break;
 	case GDK_KEY_3:
-	  insert_char = EXL;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, EXL);
 	  break;
 	case GDK_KEY_1:
-	  insert_char = TAU;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, TAU);
 	  break;
 	case GDK_KEY_6:
-	  insert_char = CTR;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, CTR);
 	  break;
 	case GDK_KEY_semicolon:
-	  insert_char = ELM;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, ELM);
 	  break;
 	case GDK_KEY_period:
-	  insert_char = NIL;
+	  pixbuf = sen_parent_get_conn_by_type (sen->parent, NIL);
 	  break;
 	case GDK_KEY_space:
 	  insert_char = " ";
@@ -852,10 +995,18 @@ sentence_key (sentence * sen, int key, int ctrl)
 	  break;
 	}
 
-      if (insert_char)
+      if (pixbuf)
 	{
-	  gtk_text_buffer_insert_at_cursor (buffer, insert_char, -1);
+	  GtkTextIter iter;
+	  GtkTextMark * mark;
+
+	  mark = gtk_text_buffer_get_insert (buffer);
+	  gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
+	  gtk_text_buffer_insert_pixbuf (buffer, &iter, pixbuf);
 	}
+
+      if (insert_char)
+	gtk_text_buffer_insert_at_cursor (buffer, insert_char, -1);
     }
   else
     {
@@ -899,7 +1050,7 @@ sentence_key (sentence * sen, int key, int ctrl)
 	  chk = (key == GDK_KEY_Left)
 	    ? gtk_text_iter_backward_char (&chk_pos)
 	    : gtk_text_iter_forward_char (&chk_pos);
-  
+
 	  break;
 	}
 
@@ -937,7 +1088,8 @@ sentence_key (sentence * sen, int key, int ctrl)
 	    }
 	  else
 	    {
-	      tmp_pos = reverse_parse_parens (sen->text, offset, NULL);
+	      tmp_pos = reverse_parse_parens (sen->text, offset,
+					      NULL);
 	      if (tmp_pos == -2)
 		return -1;
 	    }
@@ -948,39 +1100,18 @@ sentence_key (sentence * sen, int key, int ctrl)
 
 	  if (tmp_pos < 0)
 	    {
-	      gtk_text_buffer_apply_tag_by_name (buffer, "negative", &pos, &semi);
+	      gtk_text_buffer_apply_tag_by_name (buffer, "negative",
+						 &pos, &semi);
 	    }
 	  else
 	    {
-	      gtk_text_buffer_apply_tag_by_name (buffer, "hilight", &pos, &semi);
-	      /*
-	      char * cur_text, * nl_text;
-	      cur_text = gtk_text_buffer_get_text (buffer, &start, &end, TRUE);
-	      nl_text = cur_text;
-	      int line_chk = 0, fin_pos = tmp_pos;
-	      while (1)
-		{
-		  char * chk_text;
-		  chk_text = strchr (nl_text, '\n');
-		  if (!chk_text || chk_text - cur_text >= tmp_pos)
-		    {
-		      fin_pos -= (nl_text - cur_text);
-		      break;
-		    }
-
-		  line_chk++;
-		  nl_text = chk_text + 1;
-		}
-	      */
-
-	      /*
-	      gtk_text_buffer_get_iter_at_line_index (buffer, &oth_pos,
-						      line_chk, fin_pos);
-	      */
+	      gtk_text_buffer_apply_tag_by_name (buffer, "hilight",
+						 &pos, &semi);
 	      get_iter_at_index (buffer, &oth_pos, tmp_pos);
 	      semi = oth_pos;
 	      gtk_text_iter_forward_char (&semi);
-	      gtk_text_buffer_apply_tag_by_name (buffer, "hilight", &oth_pos, &semi);
+	      gtk_text_buffer_apply_tag_by_name (buffer, "hilight",
+						 &oth_pos, &semi);
 	    }
 	}
     }
@@ -996,14 +1127,13 @@ sentence_key (sentence * sen, int key, int ctrl)
  *  output:
  *    none.
  */
-
 void
 sentence_set_reference (sentence * sen, int reference, int entire_subproof)
 {
   if (reference)
-    sentence_set_bg (sen, BG_COLOR_VIOLET);
+    sentence_set_bg (sen, BG_COLOR_REF);
   else
-    sentence_set_bg (sen, BG_COLOR_WHITE);
+    sentence_set_bg (sen, BG_COLOR_DEFAULT);
 
   if (sen->subproof && entire_subproof)
     {
@@ -1020,9 +1150,9 @@ sentence_set_reference (sentence * sen, int reference, int entire_subproof)
 	    break;
 
 	  if (reference)
-	    sentence_set_bg (sub_sen, BG_COLOR_VIOLET);
+	    sentence_set_bg (sub_sen, BG_COLOR_REF);
 	  else
-	    sentence_set_bg (sub_sen, BG_COLOR_WHITE);
+	    sentence_set_bg (sub_sen, BG_COLOR_DEFAULT);
 	}
     }
 
@@ -1040,9 +1170,9 @@ void
 sentence_set_selected (sentence * sen, int selected)
 {
   if (selected)
-    sentence_set_bg (sen, BG_COLOR_RED_ORANGE);
+    sentence_set_bg (sen, BG_COLOR_SEL);
   else
-    sentence_set_bg (sen, BG_COLOR_WHITE);
+    sentence_set_bg (sen, BG_COLOR_DEFAULT);
 
   if (sen->subproof)
     {
@@ -1059,31 +1189,185 @@ sentence_set_selected (sentence * sen, int selected)
 	    break;
 
 	  if (selected)
-	    sentence_set_bg (sub_sen, BG_COLOR_RED_ORANGE);
+	    sentence_set_bg (sub_sen, BG_COLOR_SEL);
 	  else
-	    sentence_set_bg (sub_sen, BG_COLOR_WHITE);
+	    sentence_set_bg (sub_sen, BG_COLOR_DEFAULT);
 	}
     }
 
   sen->selected = selected;
 }
 
+/* Gets the correct text from a sentence text view.
+ *  input:
+ *    sen - the sentence from which to get the text.
+ *  output:
+ *    the text of the sentence, or NULL on error.
+ */
+char *
+sentence_copy_text (sentence * sen)
+{
+  char * ret_str;
+
+  GtkTextBuffer * buffer;
+  GtkTextIter start, end;
+  int text_len;
+  int i;
+
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (sen->entry));
+  gtk_text_buffer_get_bounds (buffer, &start, &end);
+
+  i = 0;
+  ret_str = (char *) calloc (1, sizeof (char));
+  CHECK_ALLOC (ret_str, NULL);
+
+  while (!gtk_text_iter_is_end (&start))
+    {
+      GdkPixbuf * pixbuf;
+      pixbuf = gtk_text_iter_get_pixbuf (&start);
+      if (pixbuf)
+	{
+	  // Determine what this is, and add the
+	  // corresponding character to ret_str;
+	  int alloc_size;
+	  unsigned char * new_char;
+	  const char * val;
+
+	  val = (char *) g_object_get_data (G_OBJECT (pixbuf),
+					    _("conn"));
+	  ret_str = (char *) realloc (ret_str,
+				      (i + CL + 1) * sizeof (char));
+	  CHECK_ALLOC (ret_str, NULL);
+	  strcpy (ret_str + i, val);
+	  i += CL;
+	}
+      else
+	{
+	  char * c;
+	  end = start;
+	  gtk_text_iter_forward_char (&end);
+
+	  c = gtk_text_iter_get_text (&start, &end);
+
+	  ret_str = (char *) realloc (ret_str, (i + 2)
+				      * sizeof (char));
+	  CHECK_ALLOC (ret_str, NULL);
+	  
+	  ret_str[i++] = *c;
+	}
+
+      gtk_text_iter_forward_char (&start);
+    }
+
+  ret_str[i] = '\0';
+
+  // Handle comments.
+
+  unsigned char * semi_str, * fin_str;
+  semi_str = strchr (ret_str, ';');
+  if (semi_str)
+    {
+      int ret_len;
+      semi_str[0] = '\0';
+      ret_len = strlen (ret_str);
+      fin_str = (unsigned char *) calloc (ret_len + 1, sizeof (char));
+      CHECK_ALLOC (fin_str, NULL);
+      memcpy (fin_str, ret_str, ret_len);
+      free (ret_str);
+      ret_str = fin_str;
+    }
+
+  return ret_str;
+}
+
+/* Sets the text of the text view of a sentence from the text.
+ *  input:
+ *    sen - the sentence of which to set the text.
+ *  output:
+ *    0 on success, -1 on memory error.
+ */
+int
+sentence_paste_text (sentence * sen)
+{
+  int i;
+
+  GtkTextBuffer * buffer;
+  GtkTextIter end;
+
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (sen->entry));
+  gtk_text_buffer_get_start_iter (buffer, &end);
+
+  for (i = 0; sen->text[i]; i++)
+    {
+      if (IS_TYPE_CONN (sen->text + i, gui_conns)
+	  || IS_TYPE_CONN (sen->text + i, cli_conns))
+	{
+	  GdkPixbuf * pix, * new_pix;
+
+	  unsigned char * conn;
+	  int len;
+
+	  if (IS_TYPE_CONN (sen->text + i, gui_conns))
+	    {
+	      if (!strncmp (sen->text + i, gui_conns.not, gui_conns.nl))
+		len = gui_conns.nl;
+	      else
+		len = gui_conns.cl;
+	    }
+	  else
+	    {
+	      if (!strncmp (sen->text + i, NOT, NL))
+		len = NL;
+	      else
+		len = CL;
+	    }
+
+	  conn = (unsigned char *) calloc (len + 1, sizeof (char));
+	  CHECK_ALLOC (conn, -1);
+	  
+	  strncpy (conn, sen->text + i, len);
+	  conn[len] = '\0';
+
+	  pix = sen_parent_get_conn_by_type (sen->parent,
+					     conn);
+	  free (conn);
+
+	  if (!pix)
+	    return -2;
+
+	  // No need to resize this, since sentence_set_font
+	  // will be called after this, and it handles it instead.
+
+	  gtk_text_buffer_insert_pixbuf (buffer, &end, pix);
+	  i += len - 1;
+	}
+      else
+	{
+	  gtk_text_buffer_insert (buffer, &end, sen->text + i, 1);
+	}
+    }
+
+  return 0;
+}
+
 /* Processes a text change in a sentence.
  *  input:
  *    sen - the sentence to process a change in.
  *  ouput:
  *    0 on success, -1 on memory error.
  */
-
 int
 sentence_text_changed (sentence * sen)
 {
+  if (sen->font_resizing)
+    return 0;
+
   sen_parent * sp = sen->parent;
   sentence_set_value (sen, VALUE_TYPE_BLANK);
 
   item_t * e_itr = ls_find (sp->everything, sen);
 
-  for (e_itr = e_itr->next; e_itr; e_itr = e_itr->next)
+  for (e_itr = sp->everything->head; e_itr; e_itr = e_itr->next)
     {
       list_t * e_refs;
       e_refs = SENTENCE (e_itr->value)->refs;
@@ -1103,14 +1387,14 @@ sentence_text_changed (sentence * sen)
       sen->sexpr = NULL;
     }
 
-  const char * text;
+  char * text;
   GtkTextBuffer * buffer;
   GtkTextIter start, end, semi;
   int text_len, old_len;
 
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (sen->entry));
   gtk_text_buffer_get_bounds (buffer, &start, &end);
-  text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+  text = sentence_copy_text (sen);
   text_len = strlen (text);
 
   int diff_pos;
@@ -1156,8 +1440,6 @@ sentence_text_changed (sentence * sen)
 	      GtkTextIter cur_pos;
 	      gtk_text_buffer_get_iter_at_mark (buffer, &cur_pos,
 						gtk_text_buffer_get_insert (buffer));
-
-
 	    }
 	}
     }
@@ -1207,10 +1489,14 @@ sentence_text_changed (sentence * sen)
 	}
     }
 
+  if (sen->text)
+    free (sen->text);
+
   sen->text = (unsigned char *) calloc (text_len + 1, sizeof (char));
   CHECK_ALLOC (sen->text, -1);
 
   strcpy (sen->text, text);
+  free (text);
   return 0;
 }
 
